/**
 * @license Copyright (c) 2003-2023, CKSource Holding sp. z o.o. All rights reserved.
 * For licensing, see LICENSE.md or https://ckeditor.com/legal/ckeditor-oss-license
 */

CKEDITOR.plugins.setLang( 'codesnippet', 'sk', {
	button: 'Vložte ukážku programového kódu',
	codeContents: 'Obsah kódu',
	emptySnippetError: 'Ukážka kódu nesmie byť prázdna.',
	language: 'Jazyk',
	title: 'Ukážka programového kódu',
	pathName: 'ukážka programového kódu'
} );
