/**
 * @license Copyright (c) 2003-2023, CKSource Holding sp. z o.o. All rights reserved.
 * For licensing, see LICENSE.md or https://ckeditor.com/legal/ckeditor-oss-license
 */

CKEDITOR.plugins.setLang( 'codesnippet', 'lt', {
	button: 'Įterpkite kodo gabaliuką',
	codeContents: 'Kodo turinys',
	emptySnippetError: 'Kodo fragmentas negali būti tusčias.',
	language: 'Kalba',
	title: 'Kodo fragmentas',
	pathName: 'kodo fragmentas'
} );
