/**
 * @license Copyright (c) 2003-2023, CKSource Holding sp. z o.o. All rights reserved.
 * For licensing, see LICENSE.md or https://ckeditor.com/legal/ckeditor-oss-license
 */

CKEDITOR.plugins.setLang( 'codesnippet', 'vi', {
	button: 'Chèn đoạn mã',
	codeContents: 'Nội dung mã',
	emptySnippetError: 'Một đoạn mã không thể để trống.',
	language: 'Ngôn ngữ',
	title: 'Đoạn mã',
	pathName: 'mã dính'
} );
