/**
 * @license Copyright (c) 2003-2023, CKSource Holding sp. z o.o. All rights reserved.
 * For licensing, see LICENSE.md or https://ckeditor.com/legal/ckeditor-oss-license
 */

CKEDITOR.plugins.setLang( 'codesnippet', 'eu', {
	button: 'Txertatu kode zatia',
	codeContents: 'Kode edukia',
	emptySnippetError: 'Kode zatiak ezin du hutsik egon.',
	language: 'Lengoaia',
	title: 'Kode zatia',
	pathName: 'kode zatia'
} );
